/* -*-gnucash-c-*-
   gnucash.c -- gnucash startup function.
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.
                                                                   
   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
                                                                   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
   (C) 1998 Rob Browning <rlb@cs.utexas.edu>
*/

#define _GNU_SOURCE
#include <stdio.h>
#include <assert.h>

#include <guile/gh.h>

#include "gnucash.h"

#define GNUCASH_DEFAULT_SHARE_DIR "/usr/share/gnucash"

static const char *
scan_argv_for_string_arg(const char *target, const int argc, char *argv[])
{
  const char *result = NULL;
  
  int i;
  for(i = 0; (!result) && (i < argc); i++)
  {
    if(strcmp(target, argv[i]) == 0)
    {
      assert((i + 1) < argc); /* FIXME: This needs to cause a usage failure */
      result = argv[i + 1];
    }
  }
  return result;
}

static SCM
gnucash_lowlev_app_init_helper() {
  const int result = gnucash_lowlev_app_init();
  return gh_long2scm(result);
}

static SCM
gnucash_lowlev_app_main_helper() {
  const int result = gnucash_lowlev_app_main();
  return gh_long2scm(result);
}

static SCM
gnucash_ui_open_file_helper(SCM name) {

  /* FIXME: We need to check name's type here.  Though this may
     eventually be handled from an automatic interface generator.
     We'l punting for now. */

  const char *c_name = gh_scm2newstr(name, NULL);
  const int result = gnucash_ui_open_file(c_name);
  return gh_long2scm(result);
}

static SCM
gnucash_ui_select_file_helper(SCM name) {
  const int result = gnucash_ui_select_file();
  return gh_long2scm(result);
}

static void gnucash_main_helper(int argc, char *argv[])
{  
  const char *share_dir = NULL;
  char *config_file = NULL;
  int result;
  
  printf("gnucash: starting up\n");
  
  gh_new_procedure("gnucash:lowlev-app-init",
                   gnucash_lowlev_app_init_helper,
                   0, 0, 0);

  gh_new_procedure("gnucash:lowlev-app-main",
                   gnucash_lowlev_app_main_helper,
                   0, 0, 0);

  gh_new_procedure("gnucash:ui-open-file",
                   gnucash_ui_open_file_helper,
                   1, 0, 0);

  gh_new_procedure("gnucash:ui-select-file",
                   gnucash_ui_select_file_helper,
                   0, 0, 0);

  /* this has to be done manually because it's the one arg we *have*
     to have (if present) before we start up guile.  It's used to find
     the guile code. */

  share_dir = scan_argv_for_string_arg("--share-dir", argc, argv);
  
  if(!share_dir)
  {
    share_dir = GNUCASH_DEFAULT_SHARE_DIR;
  }

  asprintf(&config_file, "%s/%s", share_dir, "/startup.scm");
  assert(config_file != NULL);
  
  /* We can't just use gh_eval_str here because --share-dir might have
     double quotes or other funny chars in it.  */
  {
    SCM scm_share_dir = gh_str02scm(share_dir);
    
    gh_define("gnucash:share-dir-tmp_", scm_share_dir);
    
    if(gh_load(config_file) == SCM_BOOL_F)
    {
      fprintf(stderr, "gnucash: startup file load failed");
      exit(1);
    }
  }
  
  free(config_file);
  config_file = NULL;
  
  printf("gnucash: shutting down\n");

  exit(result);
}

void
gnucash_main(int argc, char *argv[])
{
  gh_enter(argc, argv, gnucash_main_helper);
}
